@@ -60,7 +60,7 @@ class TabWidget(QTabWidget):
         self.setTabBar(bar)
         bar.tabCloseRequested.connect(self.tabCloseRequested)
         bar.tabMoved.connect(functools.partial(
-            QTimer.singleShot, 0, self._update_tab_titles))
+            QTimer.singleShot, 0, self.update_tab_titles))
         bar.currentChanged.connect(self._on_current_changed)
         bar.new_tab_requested.connect(self._on_new_tab_requested)
         self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
@@ -108,7 +108,8 @@ class TabWidget(QTabWidget):
 
         bar.set_tab_data(idx, 'pinned', pinned)
         tab.data.pinned = pinned
-        self._update_tab_title(idx)
+        self.update_tab_favicon(tab)
+        self.update_tab_title(idx)
 
     def tab_indicator_color(self, idx):
         """Get the tab indicator color for the given index."""
@@ -117,13 +118,13 @@ class TabWidget(QTabWidget):
     def set_page_title(self, idx, title):
         """Set the tab title user data."""
         self.tabBar().set_tab_data(idx, 'page-title', title)
-        self._update_tab_title(idx)
+        self.update_tab_title(idx)
 
     def page_title(self, idx):
         """Get the tab title user data."""
         return self.tabBar().page_title(idx)
 
-    def _update_tab_title(self, idx, field=None):
+    def update_tab_title(self, idx, field=None):
         """Update the tab text for the given tab.
 
         Args:
@@ -148,9 +149,13 @@ class TabWidget(QTabWidget):
         title = '' if fmt is None else fmt.format(**fields)
         tabbar = self.tabBar()
 
+        # Only change the tab title if it changes, setting the tab title causes
+        # a size recalculation which is slow.
         if tabbar.tabText(idx) != title:
             tabbar.setTabText(idx, title)
-            tabbar.setTabToolTip(idx, title)
+
+        # always show only plain title in tooltips
+        tabbar.setTabToolTip(idx, fields['title'])
 
     def get_tab_fields(self, idx):
         """Get the tab field data."""
@@ -197,20 +202,20 @@ class TabWidget(QTabWidget):
         fields['scroll_pos'] = scroll_pos
         return fields
 
-    def _update_tab_titles(self):
+    def update_tab_titles(self):
         """Update all texts."""
         for idx in range(self.count()):
-            self._update_tab_title(idx)
+            self.update_tab_title(idx)
 
     def tabInserted(self, idx):
         """Update titles when a tab was inserted."""
         super().tabInserted(idx)
-        self._update_tab_titles()
+        self.update_tab_titles()
 
     def tabRemoved(self, idx):
         """Update titles when a tab was removed."""
         super().tabRemoved(idx)
-        self._update_tab_titles()
+        self.update_tab_titles()
 
     def addTab(self, page, icon_or_text, text_or_empty=None):
         """Override addTab to use our own text setting logic.
@@ -296,6 +301,19 @@ class TabWidget(QTabWidget):
         qtutils.ensure_valid(url)
         return url
 
+    def update_tab_favicon(self, tab: QWidget):
+        """Update favicon of the given tab."""
+        idx = self.indexOf(tab)
+
+        if tab.data.should_show_icon():
+            self.setTabIcon(idx, tab.icon())
+            if config.val.tabs.tabs_are_windows:
+                self.window().setWindowIcon(tab.icon())
+        else:
+            self.setTabIcon(idx, QIcon())
+            if config.val.tabs.tabs_are_windows:
+                self.window().setWindowIcon(self.window().windowIcon())
+
 
 class TabBar(QTabBar):
 
@@ -358,7 +376,9 @@ class TabBar(QTabBar):
         # Clear _minimum_tab_size_hint_helper cache when appropriate
         if option in ["tabs.indicator.padding",
                       "tabs.padding",
-                      "tabs.indicator.width"]:
+                      "tabs.indicator.width",
+                      "tabs.min_width",
+                      "tabs.pinned.shrink"]:
             self._minimum_tab_size_hint_helper.cache_clear()
 
     def _on_show_switching_delay_changed(self):
@@ -477,7 +497,8 @@ class TabBar(QTabBar):
         Args:
             index: The index of the tab to get a size hint for.
             ellipsis: Whether to use ellipsis to calculate width
-                     instead of the tab's text.
+                      instead of the tab's text.
+                      Forced to False for pinned tabs.
         Return:
             A QSize of the smallest tab size we can make.
         """
@@ -489,14 +510,19 @@ class TabBar(QTabBar):
         else:
             icon_width = min(icon.actualSize(self.iconSize()).width(),
                              self.iconSize().width()) + icon_padding
+
+        pinned = self._tab_pinned(index)
+        if not self.vertical and pinned and config.val.tabs.pinned.shrink:
+            # Never consider ellipsis an option for horizontal pinned tabs
+            ellipsis = False
         return self._minimum_tab_size_hint_helper(self.tabText(index),
-                                                  icon_width,
-                                                  ellipsis)
+                                                  icon_width, ellipsis,
+                                                  pinned)
 
     @functools.lru_cache(maxsize=2**9)
     def _minimum_tab_size_hint_helper(self, tab_text: str,
                                       icon_width: int,
-                                      ellipsis: bool) -> QSize:
+                                      ellipsis: bool, pinned: bool) -> QSize:
         """Helper function to cache tab results.
 
         Config values accessed in here should be added to _on_config_changed to
@@ -521,6 +547,10 @@ class TabBar(QTabBar):
         height = self.fontMetrics().height() + padding_v
         width = (text_width + icon_width +
                  padding_h + indicator_width)
+        min_width = config.val.tabs.min_width
+        if (not self.vertical and min_width > 0 and
+                not pinned or not config.val.tabs.pinned.shrink):
+            width = max(min_width, width)
         return QSize(width, height)
 
     def _pinned_statistics(self) -> (int, int):
@@ -550,6 +580,12 @@ class TabBar(QTabBar):
         Return:
             A QSize.
         """
+        if self.count() == 0:
+            # This happens on startup on macOS.
+            # We return it directly rather than setting `size' because we don't
+            # want to ensure it's valid in this special case.
+            return QSize()
+
         minimum_size = self.minimumTabSizeHint(index)
         height = minimum_size.height()
         if self.vertical:
@@ -562,11 +598,6 @@ class TabBar(QTabBar):
             else:
                 width = int(confwidth)
             size = QSize(max(minimum_size.width(), width), height)
-        elif self.count() == 0:
-            # This happens on startup on macOS.
-            # We return it directly rather than setting `size' because we don't
-            # want to ensure it's valid in this special case.
-            return QSize()
         else:
             if config.val.tabs.pinned.shrink:
                 pinned = self._tab_pinned(index)
@@ -889,7 +920,7 @@ class TabBarStyle(QCommonStyle):
         # reserve space for favicon when tab bar is vertical (issue #1968)
         position = config.val.tabs.position
         if (position in [QTabWidget.East, QTabWidget.West] and
-                config.val.tabs.favicons.show):
+                config.val.tabs.favicons.show != 'never'):
             tab_icon_size = icon_size
         else:
             actual_size = opt.icon.actualSize(icon_size, icon_mode, icon_state)
