@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2017 Florian Bruhin (The-Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2018 Florian Bruhin (The-Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -127,6 +127,7 @@ def init_faulthandler(fileobj=sys.__stderr__):
         # If available, we also want a traceback on SIGUSR1.
         # pylint: disable=no-member,useless-suppression
         faulthandler.register(signal.SIGUSR1)
+        # pylint: enable=no-member,useless-suppression
 
 
 def check_pyqt_core():
@@ -171,6 +172,7 @@ def check_qt_version():
     from PyQt5.QtCore import (qVersion, QT_VERSION, PYQT_VERSION,
                               PYQT_VERSION_STR)
     from pkg_resources import parse_version
+    from qutebrowser.utils import log
     if (QT_VERSION < 0x050701 or PYQT_VERSION < 0x050700 or
             parse_version(qVersion()) < parse_version('5.7.1')):
         text = ("Fatal error: Qt >= 5.7.1 and PyQt >= 5.7 are required, "
@@ -178,6 +180,10 @@ def check_qt_version():
                                                            PYQT_VERSION_STR))
         _die(text)
 
+    if qVersion().startswith('5.8.'):
+        log.init.warning("Running qutebrowser with Qt 5.8 is untested and "
+                         "unsupported!")
+
 
 def check_ssl_support():
     """Check if SSL support is available."""
@@ -201,14 +207,14 @@ def _check_modules(modules):
             messages = ['invalid escape sequence',
                         'Flags not at the start of the expression']
             with log.ignore_py_warnings(
-                    category=DeprecationWarning,
-                    message=r'({})'.format('|'.join(messages))
+                category=DeprecationWarning,
+                message=r'({})'.format('|'.join(messages))
             ), log.ignore_py_warnings(
-                    category=PendingDeprecationWarning,
-                    module='imp'
+                category=PendingDeprecationWarning,
+                module='imp'
             ), log.ignore_py_warnings(
-                    category=ImportWarning,
-                    message=r'Not importing directory .*: missing __init__'
+                category=ImportWarning,
+                message=r'Not importing directory .*: missing __init__'
             ):
                 importlib.import_module(name)
         except ImportError as e:
@@ -260,6 +266,7 @@ def init_log(args):
 
 
 def check_optimize_flag():
+    """Check whether qutebrowser is running with -OO."""
     from qutebrowser.utils import log
     if sys.flags.optimize >= 2:
         log.init.warning("Running on optimize level higher than 1, "
